/* TWILIGHT ZONE GAME

	From 810717 (original assembler version) to 830125 in Z80 assembler.

	831025 rr begin conversion to C 0.10
	831028 rr convert some more 0.11
	831028 rr more yet 0.12
	831101 rr dictionary in room file 0.13
	831104 rr add terminal file stuff 0.14
	831106 rr simplify 0.15
	831107 rr simplify 0.16
	831108 rr misc fixup 0.17
	831109 rr more misc fixup 0.18
	831112 jd fixup to message routines 0.19        
	831114 rr further fixups, add save, load 0.20
*/

#define VRS	0
#define RLS	20

/*	Compile by: c -k -f -s500 -w60 tz	*/ 

int rc, cnt, i;
char buffer[ 25000 ];			/* buffer for reading file */
char *p, *room[ 256 ];			/* buffer pointer, rooms */
char *wlk_msg[ 24 ];			/* points to walker messages */
char inp_buf[ 34 ];			/* input buffer */
int inp_cnt = 0;			/* input buffer pointer */

char trmbuf[ 128 ];			/* storage for terminal file */

int width, height, defint, uselst, yfirst, method, addx, addy;
char *inistr, *clrscn, *clreos, *clreol, *curoff, *curon,
	*curbeg, *curmid, *curend, *hiint, *loint, *revvid, *norvid,
	*inslin, *dellin;

	/* Dictionary: words should be strung together, separated by spaces.
	Only the unique part should be included. Contained in first line
	of TZ.R. Every word including the last one must be followed by spaces.
	Incidentally, all lines of TZ.R should be terminated only by
	line feeds. */

char *dic_ptr;

/* Controls used in messages are:
	*	clear screen
	^	position cursor to message line (causes CEOL)
	\	carriage return/line feed (causes CEOL)
	|	position cursor to user input line (HEIGHT-2)
	_	include answer here (e.g. _4____)
*/

	/* The following pieces of information must be saved */

int rm_num = 0;			/* room number */
int seed;			/* random seed */
int wtchct = 0;			/* tv watch count */
int beerct = 0;			/* drink beer count */
int cratct = 0;			/* open crate count */
int pushct = 0;			/* push button count */
int pushfl = 0;			/* push button error flag */
int answer[ 5 ] = { 0, 1, 2, 3, 4 };	/* digits of answer */

struct walker {
	int inrm;		/* in room flag */
	int rsel;		/* room pointer */
	int dsel;		/* description selector */
	int locn[ 8 ];		/* rooms walker appears at */
};

	/* Woman walker descriptor */

struct walker woman = {
	0, 0, 0,
	4, 52, 64, 94, 109, 185, 201, 248 };

	/* Man walker descriptor */

struct walker man = {
	0, 0, 0,
	3, 49, 97, 105, 111, 144, 166, 175 };

	/* Rod walker descriptor */

struct walker rod = {
	0, 0, 0,
	1, 27, 61, 120, 165, 196, 180, 254 };

	/* The following are not saved */

int nu_fl = 1;			/* new room flag */
int curcol = 0;			/* cursor column */

int noun, verb;			/* word numbers */

	/* Main Program */

main()
{
	int junk;

	term_init();		/* init terminal */

	print( "*You unlock this door with the key of imagination." );
	ilpdel();
	print( "\\Beyond it is another dimension..." );
	ilpdel();
	print( "\\    ...a dimension of sound..." );
	ilpdel();
	print( "\\       ...a dimension of sight..." );
	ilpdel();
	print( "\\          ...a dimension of mind..." );
	ilpdel();
	print( "^You're moving into a land of both shadow and substance..." );
	ilpdel();
	print( "\\    ...of things and ideas..." );
	ilpdel();
	print( "\\    ...You've just crossed over..." );
	ilpdel();
	print( "\\\\          ...into the Twilight Zone." );

	/* initialize data */

	init_data();

	puts( "\n\n?Dimensions out of range at address 1423" );

	junk = 0;
	seed = 119;

	while( junk == 0 ) {
		junk = bdos( 6, 0xFF );
		seed = seed * 13;
	}

	/* synthesize solution from random number into answer[ 0..4 ] */

	/* not done yet, because save and load don't work yet. */

	while( 1 ) {
		game();		/* process game */
		input();	/* get input */
	}
}

	/* raw output of string */

puts( c )
char *c;
{
	while( *c ) putchar( *c++ );
}

term_init()
{
	int t, u;
	char *p;
	t = fopen( "TERMINAL.SYS", "r" );

	if( t == 0 ) {
		puts( "Run TERMINAL first, please." );
		exit();
	}

	p = &trmbuf[ 0 ];

	read( t, p, 128 );

	fclose( t );

	width = *p++;
	height = *p++;
	defint = *p++;
	uselst = *p++;
	yfirst = *p++;
	method = *p++;
	addx = *p++;
	addy = *p++;

	for( u = 1; u <= 15; u++ ) {
		switch( u ) {
			case 1 : inistr = p; break;
			case 2 : clrscn = p; break;
			case 3 : clreos = p; break;
			case 4 : clreol = p; break;
			case 5 : curoff = p; break;
			case 6 : curon = p; break;
			case 7 : curbeg = p; break;
			case 8 : curmid = p; break;
			case 9 : curend = p; break;
			case 10 : hiint = p; break;
			case 11 : loint = p; break;
			case 12 : revvid = p; break;
			case 13 : norvid = p; break;
			case 14 : inslin = p; break;
			case 15 : dellin = p; break;
		}

		p++; p++; p++; p++;
		p++; p++; p++; p++;
	}
}

	/* output a coordinate */

coord( c )
int c;
{
	switch( method ) {
		case '0' : putchar( c ); break;
		case '1' : {
			if( c >= 100 ) {
				putchar( '0' + c / 100 );
				c %= 100;
			}
			if( c >= 10 ) {
				putchar( '0' + c / 10 );
				c %= 10;
			}
			putchar( '0' + c );
		}
	}
}

	/* position cursor */

goxy( x, y )
int x, y;
{
	puts( curbeg );
	if( yfirst == 'Y' ) coord( y + addy );
		else coord( x + addx );
	puts( curmid );
	if( yfirst == 'Y' ) coord( x + addx );
		else coord( y + addy );
	puts( curend );
}

	/* init room list and other data */

init_data()
{
	rc = fopen( "TZ.R", "r" );
	if( rc == 0 ) {
		puts( "File not found." );
		exit( 1 );
	}

	p = &buffer[ 0 ];

	/* read entire file into buffer */

	cnt = read( rc, p, 25000 );

	fclose( rc );

	/* dictionary is first line of file */

	dic_ptr = &buffer[ 0 ];

	/* now look up indices to each string */

	p = &buffer[ 0 ];

	for( i = 0; i <= 255; i++ ) {
		while( *p != '\n' ) p++;	/* find newline */
		*p++ = '\0';			/* change to end of string */
		room[ i ] = p;			/* assign pointer */
	}
	for( i = 0; i <= 24; i++ ) {
		while( *p != '\n' ) p++;	/* find newline */
		*p++ = '\0';			/* change to end of string */
		wlk_msg[ i ] = p;		/* assign pointer */
	}
}

	/* Print Message */

print( m )
char *m;
{
	char c;
	char *k;
	int j;

	while( *m ) {
		c = *m++;

		switch( c ) {

	/* Special handling for Space.
	See if next word will fit on this line. */

			case ' ' : j = curcol;
				k = m;		/* temp pointer */

				while( j < ( width - 1 ) &&	\
					*k != ' ' && *k != '\\'	\
					&& *k != '\0' ) {
					k++;
					j++;
				}

				if( j < ( width - 1 )) {
					putchar( ' ' );
					curcol++;
				} else {
					putchar( '\n' );
					puts( clreol );
					curcol = 0;
				}

				break;

			case '\\' :		/* go to next line */
				putchar( '\n' );
				puts( clreol );
				curcol = 0;
				break;

			case '*' :		/* clear screen */
				puts( clrscn );
				puts( hiint );	
				curcol = 0;
				break;

			case '^' :		/* user line */
				goxy( 0, height / 2 - 1 );

				for( j = 1; j <= height / 2; j++ ) {
					puts( clreol );
					putchar( '\n' );
				}

				goxy( 0, height / 2 - 1 );
				curcol = 0;
				break;

			case '~' :		/* input line */
				goxy( 0, height - 1 );

				puts( loint );
				puts( revvid );

	/* clear by brute force to make line look nice */

				for( j = 1; j < width - 2; j++ )
					putchar( ' ' );
				puts( norvid );

				goxy( 1, height - 1 );
				puts( loint );
				puts( revvid );

				curcol = 0;	
				break;

	/* answer clues are imbedded in text as _1____ to _5____ */

			case '_' :		/* insert color */
				c = *m++;	/* get next char */
				switch( answer[ c & 7 - 1 ] ) {
					case 0:	puts( "red" ); break;  
					case 1: puts( "orange" ); break;
					case 2: puts( "yellow" ); break;
					case 3: puts( "green" ); break;
					case 4: puts( "blue" );	break;
				}
				while( *m++ == '_' );
				break;

			default :
				putchar( c );
				curcol++;
		}
	}

	puts( norvid );
	puts( hiint );
}

	/* Long Delay */

ilpdel()
{
	int k;
	for( k = 1; k <= 30000; k++ );
}

	/* Main Game Process */

game()
{
	char *c;

	if( ! nu_fl ) return;	/* return if haven't moved into a new room */

	nu_fl = 0;			/* reset new room flag */

	c = room[ rm_num ];		/* get initial room pointer */
	while( *c != '*' ) c++;		/* point to first asterisk */

	print( c );			/* print description */

	/* just arrived in a new room. clear all in-room flags */

	woman.inrm = 0;
	man.inrm = 0;
	rod.inrm = 0;

	/* check to see if walkers are in your room */

	if( woman.locn[ woman.rsel ] == rm_num ) {
		print( wlk_msg[ woman.dsel ] );
		woman.rsel = ++woman.rsel % 8;
		if( woman.dsel < 7 ) woman.dsel++;
		woman.inrm = 1;
	}

	if( man.locn[ man.rsel ] == rm_num ) {
		print( wlk_msg[ 8 + man.dsel ] );
		man.rsel = ++man.rsel % 8;
		if( man.dsel < 7 ) man.dsel++;
		man.inrm = 1;
	}

	if( rod.locn[ rod.rsel ] == rm_num ) {
		print( wlk_msg[ 16 + rod.dsel ] );
		rod.rsel = ++rod.rsel % 8;
		if( rod.dsel < 7 ) rod.dsel++;
		rod.inrm = 1;
	}

	print( "~    What do you do? " );
}

	/* Input Process */

input()
{
	char c;

	c = bdos( 6, -1 );
	if( c == 0 ) return;		/* quit if no input */

	goxy( inp_cnt + 22, height - 1 );

	switch( c ) {
		case '\r' :
			parse();		/* get command */
			print( "~      What do you do? " ); /* re-prompt */
			inp_cnt = 0;
			break;
		case '\b' :
			if( ! inp_cnt ) break;	/* backspace */
			inp_cnt--;
			puts( "\b \b" );
			break;

		default :
			if( c < ' ' || inp_cnt > 30 ) break;
			if( c >= 'A' && c <= 'Z' )
				c += 32;	/* convert lower */
			putchar( c );		/* duplex */
			inp_buf[ inp_cnt++ ] = c;
	}
}

	/* Get word */

get_word( p )
char *p;
{
	int k, match;
	char *q, *d;
 
	if( ! *p ) return( 0 );

	k = 0;					/* word counter */
	d = dic_ptr;				/* index into dict */

	while( *d ) {
		q = p;				/* point to begin of word */
		match = 1;			/* pretend we have a match */

		k++;				/* count a word */ 

		while( *d != ' ' ) {		/* compare this word */
			if( *q++ != *d++ ) match = 0;
		}				/* d points to space */

		if( match ) return( k );	/* got a match */
		d++;				/* skip the space */
	}

	return( 0 );	/* came to end of dictionary */
}

	/* Parse Command */

parse()
{
	char *p_ptr;

	if( ! inp_cnt ) return;			/* empty line input */

	inp_buf[ ++inp_cnt ] = ' ';		/* add space to input */
	inp_buf[ ++inp_cnt ] = '\0';		/* terminate it */

	noun = 0;
	verb = 0;

	p_ptr = &inp_buf[ 0 ];			/* point to first word */

	verb = get_word( p_ptr );		/* get verb */

	while( *p_ptr != ' ' ) p_ptr++;		/* find blank */
	while( *p_ptr == ' ' ) p_ptr++;		/* find non-blank */

	noun = get_word( p_ptr );		/* get noun if any */

	switch( verb ) {
		case 7 : go( noun ); break;

		case 8 : if( noun == 29 && rm_num == 63 ) taktrn();
				else print( "^You can't take it with you." );
			break;		

		case 9 : exit();	/* quit game */
 
		case 10 : if( noun == 11 ) {
				switch( rm_num ) {
					case 7 : call11(); break;
					case 11 : call7(); break;
					case 174 :
					case 222 :
					case 249 : busy(); break;
					default : cant();
				}
			}
			else cant();
			break;
		case 13 : switch( noun ) {
			case 14 : menu( rm_num ); break;

			case 15 : if( rm_num == 7 || rm_num == 174 \
					|| rm_num == 222 || rm_num == 249 )
					sign();
					else cant();
				break;

			case 16 : if( rm_num == 7 || rm_num == 174 \
					|| rm_num == 222 || rm_num == 249 )
					book();
					else cant();
				break;

			case 17 : if( rm_num == 100 ) form();
				else cant();
				break;
			case 24 : writing( rm_num ); break;

			case 25 : if( rm_num == 29 ) inst();
				else cant();
				break;

			case 39 : plates( rm_num ); break;

			case 42 : paper( rm_num ); break;

			default : cant(); break;
			}
			break;

		case 20 : if( noun == 21 && rm_num == 13 ) buyber();
			else cant();
			break;

		case 22 : switch( noun ) {
				case 18 : if( rm_num == 100 ) blot();
					else cant();
					break;
				case 41 : if( rm_num == 21 ) excrat();
					else cant();
					break;
				case 31 : if( rm_num == 163 ) examtv();
					else cant();
					break;
				case 23 : if( rm_num == 198 ) exhand();
					else cant();
					break;
				case 54 :
				case 49 : if( rm_num == 58 ) pool();
					else cant();
					break;
				case 57 : if( rm_num == 38 || \
						rm_num == 66 || \
						rm_num == 178 ) {
						exbed(); break;
					}
				default : print( "^Nothing unusual." );
					break;
			}
			break;

		case 30 : if( noun == 31 && rm_num == 163 ) wtchtv();
				else cant();
				break;
		case 37 : if( rm_num == 128 ) push();
				else cant();
				break;
		case 38 : switch( noun ) {		/* LOOK */
				case 12 : if( rm_num == 0 || rm_num == 66 \
						|| rm_num == 106 \
						|| rm_num == 178 \
						|| rm_num == 255 ) {
						seeslf();
						break;
					}
					if( rm_num == 38 ) {
						seecow();
						break;
					}
					if( rm_num >= 223 && rm_num <= 231 ) {
						mirhse( rm_num );
						break;
					}
					cant();
					break;
				case 54 :
				case 49 : if( rm_num == 58 ) pool();
					else cant();
					break;
				default : nu_fl = 1; break;
			}
			break;		

		case 40 : switch( noun ) {
				case 41 : if( rm_num == 21 ) opcrat();
					else cant(); break;
				case 43 : if( rm_num == 152 ) opsafe();
					else cant(); break;
				case 52 : if( rm_num == 220 ) opgrat();
					else cant(); break;
				default : print( "^There's nothing to open." );
					break;
			}
			break;

		case 44 : if( noun == 45 && rm_num == 152 ) dlcomb();
			else cant(); break;
		case 46 : savgam(); break;
		case 47 : lodgam(); break;

		case 48 : switch( noun ) {
				case 21 : if( rm_num == 13 ) drbeer();
					else cant();
					break;
				case 49 : if( rm_num == 9 || rm_num == 221 ) {
						drwatr();
						break;
					}
					if( rm_num == 58 ) echh();
					else cant();
					break;
				case 54 : if( rm_num == 58 ) echh();
					else cant();
					break;
				default : cant(); break;
			}
			break;

		case 50 : if( noun == 51 && rm_num == 14 ) ridhor();
				else cant();
			break;
		case 55 : say();		/* speak to walkers */
			break;	

	/* synonyms for go */

		case 64 : go( 1 ); break;
		case 65 : go( 2 ); break;
		case 66 : go( 3 ); break;
		case 67 : go( 4 ); break;
		case 68 : go( 5 ); break; 
		case 69 : go( 6 ); break;
		case 70 : go( 27 ); break;
		case 71 : go( 28 ); break;

		default : cant();
	}
}

savgam()
{
	int fc, i;

	fc = fopen( "TZ.SAV", "wb" );

	if( fc == 0 ) {
		print( "^Unable to create output file." );
		return;
	}

	putc( rm_num, fc );	/* write room number */
	putc( seed % 256, fc );
	putc( seed / 256, fc );	/* random seed */
	putc( wtchct, fc );	/* tv watch count */
	putc( beerct, fc );	/* drink beer count */
	putc( cratct, fc );	/* open crate count */
	putc( pushct, fc );	/* push button count */
	putc( pushfl, fc );	/* push button error flag */

	for( i = 0; i < 5; i++ ) putc( answer[ i ], fc );
				/* digits of answer */

	putc( woman.inrm, fc );
	putc( woman.rsel, fc );
	putc( woman.dsel, fc );

	putc( man.inrm, fc );
	putc( man.rsel, fc );
	putc( man.dsel, fc );

	putc( rod.inrm, fc );
	putc( rod.rsel, fc );
	putc( rod.dsel, fc );

	fclose( fc );

	print( "^Game has been saved." );
}

lodgam()
{
	int fc, i;

	fc = fopen( "TZ.SAV", "rb" );

	if( fc == 0 ) {
		print( "^Unable to open input file." );
		return;
	}

	rm_num = getc( fc );		/* write room number */
	seed = getc( fc );
	seed += 256 * getc( fc );	/* random seed */
	wtchct = getc( fc );		/* tv watch count */
	beerct = getc( fc );		/* drink beer count */
	cratct = getc( fc );		/* open crate count */
	pushct = getc( fc );		/* push button count */
	pushfl = getc( fc );		/* push button error flag */

	for( i = 0; i < 5; i++ ) answer[ i ] = getc( fc );
				/* digits of answer */

	woman.inrm = getc( fc );
	woman.rsel = getc( fc );
	woman.dsel = getc( fc );

	man.inrm = getc( fc );
	man.rsel = getc( fc );
	man.dsel = getc( fc );

	rod.inrm = getc( fc );
	rod.rsel = getc( fc );
	rod.dsel = getc( fc );

	fclose( fc );

	nu_fl = 1;			/* flag new room */
}

	/* Go Routine */

go( n )
int n;
{
	char dirc, *c;

	switch( n ) {
		case 1 : dirc = 'n'; break;	/* get direction character */
		case 2 : dirc = 's'; break;
		case 3 : dirc = 'e'; break;
		case 4 : dirc = 'w'; break;
		case 5 : dirc = 'u'; break;
		case 6 : dirc = 'd'; break;
		case 27 : dirc = 'i'; break;
		case 28 : dirc = 'o'; break;
		case 19 : dirc = 'b'; break;	/* below */
		case 26 : dirc = 'z'; break;	/* sleep */
		default : dirc = '?'; break;
	}

	c = room[ rm_num ];		/* get pointer to room */

	while( *c != '*' ) {
		if( *c++ == dirc ) {
			rm_num = convert( c );
			nu_fl = 1;	/* inform game() */
			return;
		}
	}

	/* fell through, direc must be bad */

	print( "^You can't go that way." );
}

int convert( c )
char *c;
{
	int r;

	r = 0;

	while( *c >= '0' && *c <= '9' )
		r = 10 * r + *c++ - '0';

	return( r );
}

	/* Simply Cannot Do Something */

cant()
{
	print( "^You cannot do that here." );
}

	/* Speak to the Walkers */

say()
{
	if( woman.inrm ) {
		switch( noun ) {
			case 56 :
	print( "^'Hi,' she smiles back. 'Have you found anything?'" );
				break;
			case 45 : print( "*Aileen has shot you." );
				exit( 0 );
			default :
	print( "^The woman gives you a puzzled look." );
	print( " 'What are you trying to say?'" );
		}
		return;
	}

	if( man.inrm ) {
		print( "^You'd like to talk to him, sure, but he''s already" );
		print( " gone." );
		return;
	}

	if( rod.inrm ) {
		print( "^The man in the dark suit pays no attention to" );
		print( " anything you say." );
		return;
	}

	print( "^There is no one here to speak to." );
}

	/* Examine Bed */

exbed()
{
	print( "^What are beds for?" );
}

	/* Marsh */

pool()
{
	print( "^You see yourself looking down into the pool..." );
	print( " Could it mean that YOU are the MARSH-IAN?" );
}

echh()
{
	print( "^ECHH! Tastes AWFUL!" );
}

	/* Phone Booths */

busy()
{
	print( "^The phone is busy." );
}

call7()
{
	rm_num = 7;
	nu_fl = 1;
}

call11()
{
	rm_num = 11;
	nu_fl = 1;
}

sign()
{
	print( "^'No dime needed to call operator'." );
}

book()
{
	print( "^'Let your fingers do the walking . . .'" );
}

	/* Safe (a Red Herring) */

opsafe()
{
	print( "^It's locked. If you knew the combination, you" );
	print( " could dial it." );
}

dlcomb()
{
	print( "^The heavy steel door of the safe swings open," );
	print( " revealing... nothing, save a small piece of paper." );
}

paper( r )
int r;
{
	switch( r ) {
		case 86 :
print( "^The paper says simply, 'Combination: 22-41-66.'" );
			break;
		case 152 : print( "^'Thought it would be that easy?'" );
			break;
		default : cant();
	}
}

form()
{
	print( "^The Racing Form is a newspaper-like publication" );
	print( " which lists local horse races and shows calculated" );
	print( " odds on each horse. Gamblers often read these" );
	print( " before placing bets." );
	print( "\\There is an inkblot on this form, which appears" );
	print( " to have inadvertantly dripped from the pen." );
}

blot()
{
	print( "^It looks like a crab, or maybe, ..." );
	print( "\Anyway, the blot has landed next to a horse" );
	print( " in the third race named 'Royal _3____'. Might be a " );
	print( "good bet!" );
}

	/* Mirrors */

seeslf()
{
	print( "^You see yourself standing in the hotel room." );
}

seecow()
{
	print( "^You see yourself standing in the hotel room -- but dressed \
up in dusty Old West clothing!" );
}

mirhse( i )
int i;
{
	print( "^You see yourself in a state of" );
	switch( i ) {
		case 223 : print( " confidence." ); break;
		case 224 : print( " frustration." ); break;
		case 225 : print( " thought." ); break;
		case 226 : print( " indecision." ); break;
		case 227 : print( " desperation." ); break;
		case 228 : print( " anxiety." ); break;
		case 229 : print( " reflection." ); break;
		case 230 : print( " agitation." ); break;
		case 231 : print( " surprise!" ); break;
	}
}

menu( r )
int r;
{
	switch( r ) {
		case 9 : print( "^'House Specialties:" );
print( "\\Spare Ribs ... 6.98     Red Herring ... 5.00" );
print( "\\Food for Thought:" );
print( " You must find something in this world and take" );
print( " it somewhere. Watch for clues as to what it is." );
print( " Remember, Information is the most valuable of" );
print( " all possessions.'" );
			break;
		case 221 : print( "^'House Specialties:" );
print( "\\Spare Ribs ... 6.98    Blue Trout ... 2.65" );
print( "\\Food for Thought: Read the Plate." );
			break;
		default: print( "^Don't see one here." );
	}
}

drwatr()
{
	print( "^Ahh... you were getting thirsty!" );
}

	/* Crate */

excrat()
{
	if (cratct == 0 )
		print( "^It looks as though it could be opened..." );
	else {
		print(  "^Very odd indeed! The crate has a smaller" );
		print( " crate inside it ...\\within that one is a" );
		print( " ... wax hand? Maybe part of a statue?" );
	}
}

opcrat()
{
	if( ++cratct > 1 )
		print( "^It's already open!" );
	else
		print( "^ "); 
}

writing( r )
int r;
{
	switch( r ) {
		case 21 : print( "^'Buy a beer at the saloon.'" );
			break;
		case 214 : print( "^'The Twilight Zone Game is" );
print( " Copyright (C) 1982, 1983 by R. Rodman. All rights reserved.'" );
			break;
		default : print( "^Don't see any." );
	}
}

inst()
{
	print( "^'The key to the Zone is a multi-digit code. Once having" );
	print( " determined it, you may leave the Zone if you use it in" );
	print( " the proper way. Be careful not to disclose it to the" );
	print( " wrong person!'" );
}

	/* Buy Beer */

buyber()
{
	switch( ++beerct ) {
		case 1 :
			print( "^You strike up a conversation with the" );
			print( " bartender.\'Let me give you a hint,' he" );
			print( " says. 'When you get to Pleasantville," );
			print( " take a ride on the train.'" );
			print( " He then goes back to washing glasses." );
			break;
		default :
			print( "^Without saying a word, he slides a glass" );
			print( " of beer over to you." );
	}
}

drbeer()
{
	print( "^Ahh... Really hits the spot!" );
}

ridhor()
{
	print( "^The horse won''t go anywhere, no matter what you do." );
}

taktrn()
{
	print( "^The train pulls up and you get on. It rolls out and" );
	print( " travels past the Pleasantville water tower. Then, it" );
	print( " rolls past five abandoned automobiles. The second one" );
	print( " looks like an old race car; it is painted a strange" );
	print( " _2____ color, and on it is a number 6." );
	print( " The train continues past some cornfields" );
	print( " and crosses the gravel road entering Pleasantville." );
	print( " Finally, it arrives back at the station again." );
}

taknot()
{
	print( "^You can''t take it with you." );
}

	/* Read License Plates */

plates( i )
int i;
{
	switch( i ) {
		case 9 :
		case 221 :
print( "^There is nothing written on it. Perhaps a different kind of plate" );
print( " might prove more interesting!" );
			break;
		case 205 : print( "^ 'B46 317'." ); break;
		case 206 : print( "^ 'GJJ-165'." ); break;
		case 207 : print( "^ '301 BFJ'." ); break;
		case 208 : print( "^ 'SQR-602'." ); break;
		case 209 : print( "^ 'F65-G37'." ); break;
		case 210 : print( "^ '1225348'." ); break;
		case 211 : print( "^ '421 JK'." ); break;
		case 212 : print( "^ 'JKL-123'." ); break;
		case 213 : print( "^ 'DIGIT 5'." ); break;
		default : cant(); break;
	}
}

opgrat()
{
	print( "^Oh boy! Maybe there''s a stairway leading to a" );
	print( " dome, and a cage, and a bird, and a rusty black" );
	print( " rod with a star on the end!" );
	print( " \... Nuts, it won''t open.$" );
}

	/* Watch TV */

examtv()
{
	print( "^It is set to channel 10." );
}

wtchtv()
{
	if( wtchct < 5 ) wtchct++;
	switch( wtchct ) {
		case 1 : {
			print( "^There are slanted, almost horizontal " );
			print( "lines on the screen, as though the " );
			print( "set is out of repair. The sound is garbled," );
			print( "apparently a commercial." );
			break;
		}
		case 2 : {
			print( "^The picture has cleared up and now shows " );
			print( "YOU! You are walking across a desert past " );
			print( "an old car." );
			break;
		}
		case 3 : {
			print( "^The screen shows you near a burned-out " );
			print( "place in the desert, crouched over a " );
			print( "crumpled body." );
			break;
		}
		case 4 : {
			print( "^The screen shows you closely examining a " );
			print( "hand, as though there were something odd " );
			print( "about it!" );
			break;
		}
		case 5 : {
			print( "^The screen is filled with slanted, almost " );
			print( "horizontal lines." );
			break;
		}
	}
}

exhand()
{
	print( "^The hand is wearing a bright _1____ ring on its index" );
	print( " finger." );
}

	/* push button */

push()
{
	if( noun < 32 || noun > 36 ) {
		cant();
		return;
	}

	if( noun - 32 != answer[ pushct++ ] ) pushfl = 1;

	if( pushct < 5 ) {
		print( "^There is a faint click." );
		return;
	}

	if( pushfl ) {
		pushct = 0;		/* incorrect, reset everything */
		pushfl = 0;
		print( "^There is a beeping sound." );
		return;
	}

	print( "*A blinding light surrounds you momentarily, and" );
	print( " when it subsides, you find yourself in a beautiful" );
	print( " meadow. Children play nearby, and the sun glints" );
	print( " off the snow on rugged mountain peaks in the far" );
	print( " distance. You have escaped the Zone and returned" );
	print( " to your own dimension. Enjoy it!" );
	exit( 0 );
}

#include "stdlib.c"
